/*
 *  linux/arch/mips/kernel/setup.c
 *
 *  Copyright (C) 1995  Linus Torvalds
 *  Copyright (C) 1995, 1996  Ralf Baechle
 *  Copyright (C) 1996  Stoned Elipot
 */
#include <linux/config.h>
#include <linux/errno.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/stddef.h>
#include <linux/string.h>
#include <linux/unistd.h>
#include <linux/ptrace.h>
#include <linux/malloc.h>
#include <linux/user.h>
#include <linux/utsname.h>
#include <linux/a.out.h>
#include <linux/tty.h>
#ifdef CONFIG_BLK_DEV_RAM
#include <linux/blk.h>
#endif

#include <asm/asm.h>
#include <asm/bootinfo.h>
#include <asm/io.h>
#include <asm/vector.h>
#include <asm/segment.h>
#include <asm/stackframe.h>
#include <asm/system.h>

/*
 * How to handle the machine's features
 */
struct feature *feature;

#ifdef CONFIG_DECSTATION
void decstation_handle_int(void);
/*
 * FIXME: Don't have any of the goo required to access fd or rtc etc.
 */
struct feature decstation_feature = {
	decstation_handle_int
};
#endif
#ifdef CONFIG_DESKSTATION_RPC44
void deskstation_rpc44_handle_int(void);
/*
 * How to access the floppy controller's ports
 */
unsigned char deskstation_rpc44_fd_inb(unsigned int port);
void deskstation_rpc44_fd_outb(unsigned char value, unsigned int port);
/*
 * How to access the floppy DMA functions.
 */
void deskstation_rpc44_fd_enable_dma(void);
void deskstation_rpc44_fd_disable_dma(void);
int deskstation_rpc44_fd_request_dma(void);
void deskstation_rpc44_fd_free_dma(void);
void deskstation_rpc44_fd_clear_dma_ff(void);
void deskstation_rpc44_fd_set_dma_mode(char mode);
void deskstation_rpc44_fd_set_dma_addr(unsigned int a);
void deskstation_rpc44_fd_set_dma_count(unsigned int count);
int deskstation_rpc44_fd_get_dma_residue(void);
void deskstation_rpc44_fd_enable_irq(void);
void deskstation_rpc44_fd_disable_irq(void);
void deskstation_rpc44_fd_cacheflush(unsigned char *addr, unsigned int size);
/*
 * How to access the RTC functions.
 */
unsigned char deskstation_rpc44_rtc_read_data(void);
void deskstation_rpc44_rtc_write_data(unsigned char data);

struct feature deskstation_rpc44_feature = {
	deskstation_rpc44_handle_int,
/*
	 * How to access the floppy controller's ports
	 */
	deskstation_rpc44_fd_inb,
	deskstation_rpc44_fd_outb,
	/*
	 * How to access the floppy DMA functions.
	 */
	deskstation_rpc44_fd_enable_dma,
	deskstation_rpc44_fd_disable_dma,
	deskstation_rpc44_fd_request_dma,
	deskstation_rpc44_fd_free_dma,
	deskstation_rpc44_fd_clear_dma_ff,
	deskstation_rpc44_fd_set_dma_mode,
	deskstation_rpc44_fd_set_dma_addr,
	deskstation_rpc44_fd_set_dma_count,
	deskstation_rpc44_fd_get_dma_residue,
	deskstation_rpc44_fd_enable_irq,
	deskstation_rpc44_fd_disable_irq,
	deskstation_rpc44_fd_cacheflush,
	/*
	 * How to access the RTC functions.
	 */
	deskstation_rpc44_rtc_read_data,
	deskstation_rpc44_rtc_write_data
};
#endif
#ifdef CONFIG_DESKSTATION_TYNE
void deskstation_tyne_handle_int(void);
void isa_outb(unsigned char value, unsigned int port);
unsigned char isa_inb(unsigned int port);
struct feature deskstation_tyne_feature = {
	deskstation_tyne_handle_int,
	isa_inb,
	isa_outb,
};
#endif
/*
 * All the Jazz family machines use this same structure
 */
#ifdef CONFIG_MIPS_JAZZ
void jazz_handle_int(void);
/*
 * How to access the floppy controller's ports
 */
unsigned char jazz_fd_inb(unsigned int port);
void jazz_fd_outb(unsigned char value, unsigned int port);
/*
 * How to access the floppy DMA functions.
 */
void jazz_fd_enable_dma(void);
void jazz_fd_disable_dma(void);
int jazz_fd_request_dma(void);
void jazz_fd_free_dma(void);
void jazz_fd_clear_dma_ff(void);
void jazz_fd_set_dma_mode(char mode);
void jazz_fd_set_dma_addr(unsigned int a);
void jazz_fd_set_dma_count(unsigned int count);
int jazz_fd_get_dma_residue(void);
void jazz_fd_enable_irq(void);
void jazz_fd_disable_irq(void);
void jazz_fd_cacheflush(unsigned char *addr, unsigned int size);
/*
 * How to access the RTC functions.
 */
unsigned char jazz_rtc_read_data(void);
void jazz_rtc_write_data(unsigned char data);

struct feature jazz_feature = {
	jazz_handle_int,
	/*
	 * How to access the floppy controller's ports
	 */
	jazz_fd_inb,
	jazz_fd_outb,
	/*
	 * How to access the floppy DMA functions.
	 */
	jazz_fd_enable_dma,
	jazz_fd_disable_dma,
	jazz_fd_request_dma,
	jazz_fd_free_dma,
	jazz_fd_clear_dma_ff,
	jazz_fd_set_dma_mode,
	jazz_fd_set_dma_addr,
	jazz_fd_set_dma_count,
	jazz_fd_get_dma_residue,
	jazz_fd_enable_irq,
	jazz_fd_disable_irq,
	jazz_fd_cacheflush,
	/*
	 * How to access the RTC functions.
	 */
	jazz_rtc_read_data,
	jazz_rtc_write_data
};
#endif

/*
 * Tell us the machine setup..
 */
char wait_available;		/* set if the "wait" instruction available */

/*
 * Bus types ..
 */
int EISA_bus = 0;

/*
 * Setup options
 */
struct drive_info_struct drive_info;
struct screen_info screen_info = SCREEN_INFO;

/*
 * setup informations
 *
 * These are intialized so they are in the .data section
 */
unsigned long mips_memory_upper = KSEG0; /* this is set by kernel_entry() */
unsigned long mips_cputype = CPU_UNKNOWN;
unsigned long mips_machtype = MACH_UNKNOWN;
unsigned long mips_machgroup = MACH_GROUP_UNKNOWN;
unsigned long mips_tlb_entries = 48; /* Guess which CPU I've got :) */
unsigned long mips_vram_base = KSEG0;
#if defined( CONFIG_DESKSTATION_TYNE )
unsigned long mips_dma_cache_size = 0;
unsigned long mips_dma_cache_base = KSEG0;
#endif /* CONFIG_DESKSTATION_TYNE */

unsigned char aux_device_present;
extern int root_mountflags;
extern int _end;

extern char empty_zero_page[PAGE_SIZE];

/*
 * This is set up by the setup-routine at boot-time
 */
#define PARAM	empty_zero_page
#if 0
#define ORIG_ROOT_DEV (*(unsigned short *) (PARAM+0x1FC))
#define AUX_DEVICE_INFO (*(unsigned char *) (PARAM+0x1FF))
#endif
#define LOADER_TYPE (*(unsigned char *) (PARAM+0x210))
#define KERNEL_START (*(unsigned long *) (PARAM+0x214))
#define INITRD_START (*(unsigned long *) (PARAM+0x218))
#define INITRD_SIZE (*(unsigned long *) (PARAM+0x21c))

static char command_line[CL_SIZE] = { 0, };
       char saved_command_line[CL_SIZE];

/*
 * isa_slot_offset is the address where E(ISA) busaddress 0 is is mapped
 * for the processor.
 */
unsigned long isa_slot_offset;

/*
 * Parse the tags present in upper memory to find out
 * a pecular one.
 *
 * Parameter: type - tag type to find
 * 
 * returns  : NULL  - failure
 *            !NULL - pointer on the tag structure found 
 */
tag *
bi_TagFind(enum bi_tag type)
{
  tag* t;
  t = (tag*)(mips_memory_upper - sizeof(tag));
  while((t->tag != tag_dummy) && (t->tag != type))
    {
      t = (tag*)(NEXTTAGPTR(t));
    }
  if (t->tag == tag_dummy)		/* tag not found */
    {
      return (tag*)NULL;
    }
  return t;
}

/*
 * Snarf from the tag list in memory end some tags needed
 * before the kernel reachs setup_arch()
 *
 * add yours here if you want to, but *beware*: the kernel var
 * that will hold the values you want to snarf have to be
 * in .data section of the kernel, so initialized in to whatever
 * value in the kernel's sources.
 */
void bi_EarlySnarf(void)
 {
  tag* atag;
  
  /* for wire_mappings() */
  atag = bi_TagFind(tag_machtype);
  if (atag)
    {
      memcpy(&mips_machtype, TAGVALPTR(atag), atag->size);
    }
  else
    {
      /* useless for boxes without text video mode but....*/
      panic("machine type not specified by bootloader");
    }

  /* for tlbflush() */
  atag = bi_TagFind(tag_tlb_entries);
  if (atag)
    memcpy(&mips_tlb_entries, TAGVALPTR(atag), atag->size);
  else
    {
      /* useless for boxes without text video mode but....*/
      panic("number of TLB entries not specified by bootloader");
    }
  return;
}


void setup_arch(char **cmdline_p,
	unsigned long * memory_start_p, unsigned long * memory_end_p)
{
	unsigned long memory_start, memory_end;
	tag* atag;

	/* Perhaps a lot of tags are not getting 'snarfed' - */
	/* please help yourself */

	atag = bi_TagFind(tag_machtype);
	memcpy(&mips_machtype, TAGVALPTR(atag), atag->size);

	atag = bi_TagFind(tag_cputype);
	memcpy(&mips_cputype, TAGVALPTR(atag), atag->size);

	atag = bi_TagFind(tag_machgroup);
	memcpy(&mips_machgroup, TAGVALPTR(atag), atag->size);

	atag = bi_TagFind(tag_vram_base);
	memcpy(&mips_vram_base, TAGVALPTR(atag), atag->size);

#if defined( CONFIG_DESKSTATION_TYNE )
	atag = bi_TagFind(tag_dma_cache_size);
	memcpy(&mips_dma_cache_size, TAGVALPTR(atag), atag->size);

	atag = bi_TagFind(tag_dma_cache_base);
	memcpy(&mips_dma_cache_base, TAGVALPTR(atag), atag->size);
#endif /* CONFIG_DESKSTATION_TYNE */

	switch(mips_machtype)
	{
#ifdef CONFIG_DECSTATION
	case MACH_DECSTATION:
		feature = &decstation_feature;
		break;
#endif
#ifdef CONFIG_DESKSTATION_RPC44
	case MACH_DESKSTATION_RPC44:
		mips_memory_upper = KSEG0 + (32 << 20); /* xxx fixme imp */
		feature = &deskstation_rpc44_feature;
		isa_slot_offset = 0xa0000000;
		request_region(0x00,0x20,"dma1");
		request_region(0x40,0x20,"timer");
		request_region(0x70,0x10,"rtc");
		break;
#endif
#ifdef CONFIG_DESKSTATION_TYNE
	case MACH_DESKSTATION_TYNE:
		feature = &deskstation_tyne_feature;
		isa_slot_offset = 0xa0000000;
		request_region(0x00,0x20,"dma1");
		request_region(0x40,0x20,"timer");
		request_region(0x70,0x10,"rtc");
		break;
#endif
#ifdef CONFIG_MIPS_JAZZ
	case MACH_ACER_PICA_61:
	case MACH_MIPS_MAGNUM_4000:
		feature = &jazz_feature;
		isa_slot_offset = 0xe3000000;
		request_region(0x00,0x20,"dma1");
		request_region(0x40,0x20,"timer");
		/* The RTC is outside the port address space */
		break;
#endif
	default:
		panic("Unsupported architecture");
	}

#if 0
 	ROOT_DEV = to_kdev_t(ORIG_ROOT_DEV);
#else
#ifdef CONFIG_ROOT_NFS
	ROOT_DEV = to_kdev_t(0x00ff);	/* NFS */
#elif CONFIG_BLK_DEV_FD
	ROOT_DEV = to_kdev_t(0x021c);	/* fd0H1440 */
#else
	ROOT_DEV = to_kdev_t(0x0101);	/* ram */ 
#endif
#endif

	atag = bi_TagFind(tag_drive_info);
	memcpy(&drive_info, TAGVALPTR(atag), atag->size);
#if 0
	aux_device_present = AUX_DEVICE_INFO;
#endif

	memory_end = mips_memory_upper;
	/*
	 * Due to prefetching and similar mechanism the CPU sometimes
	 * generates addresses beyond the end of memory.  We leave the size
	 * of one cache line at the end of memory unused to make shure we
	 * don't catch this type of bus errors.
	 */
	memory_end -= 32;
	memory_end &= PAGE_MASK;

#ifdef CONFIG_BLK_DEV_RAM
	rd_image_start = RAMDISK_FLAGS & RAMDISK_IMAGE_START_MASK;
	rd_prompt = ((RAMDISK_FLAGS & RAMDISK_PROMPT_FLAG) != 0);
	rd_doload = ((RAMDISK_FLAGS & RAMDISK_LOAD_FLAG) != 0);
#endif
#ifdef CONFIG_MAX_16M
	/*
	 * There is a quite large number of different PC chipset based boards
	 * available and so I include this option here just in case ...
	 */
	if (memory_end > PAGE_OFFSET + 16*1024*1024)
		memory_end = PAGE_OFFSET + 16*1024*1024;
#endif

	atag = bi_TagFind(tag_mount_root_rdonly);
	if (atag)
	  root_mountflags |= MS_RDONLY;

	/*
	 * On the deskstation rpc44, for the moment, only the kernel is
	 * below 1M.  xxx imp fixme  MILO should set these, but doesn't
	 * seem to be.
	 */
	if (mips_machtype == MACH_DESKSTATION_RPC44)
		memory_start = (unsigned long) KSEG0 + (1 << 20);
	else
		memory_start = (unsigned long) &_end;

	atag = bi_TagFind(tag_command_line);
	if (atag)
	  memcpy(&command_line, TAGVALPTR(atag), atag->size);	  

	memcpy(saved_command_line, command_line, CL_SIZE);
	saved_command_line[CL_SIZE-1] = '\0';

	*cmdline_p = command_line;
	*memory_start_p = memory_start;
	*memory_end_p = memory_end;

#ifdef CONFIG_BLK_DEV_INITRD
	if (LOADER_TYPE) {
		initrd_start = INITRD_START;
		initrd_end = INITRD_START+INITRD_SIZE;
		if (initrd_end > memory_end) {
			printk("initrd extends beyond end of memory "
			       "(0x%08lx > 0x%08lx)\ndisabling initrd\n",
			       initrd_end,memory_end);
		initrd_start = 0;
	}
}
#endif
}
