#include <linux/mm.h>
#include <asm/cachectl.h>
#include <asm/mipsconfig.h>
#include <asm/page.h>
#include <asm/pgtable.h>

/*
 * Initialize new page directory with pointers to invalid ptes
 */
void pgd_init(unsigned long page)
{
	unsigned long dummy1, dummy2;

	/*
	 * This version is optimized for the R6000.  We generate dirty lines
	 * in the datacache, overwrite these lines with zeros and then flush
	 * the cache.  Sounds horribly complicated but is just a trick to
	 * avoid unnecessary loads of from memory and uncached stores which
	 * are very expensive.  Not tested yet as the R6000 is a rare CPU only
	 * available in SGI machines and I don't have one.
	 */
	__asm__ __volatile__(
		".set\tnoreorder\n"
		"1:\t"
#ifndef CONF_DISABLE_KSEG0_CACHING
		"cache\t%5,(%0)\n\t"
#endif
		"sw\t%2,(%0)\n\t"
		"sw\t%2,4(%0)\n\t"
		"sw\t%2,8(%0)\n\t"
		"sw\t%2,12(%0)\n\t"
#ifndef CONF_DISABLE_KSEG0_CACHING
		"cache\t%5,16(%0)\n\t"
#endif
		"sw\t%2,16(%0)\n\t"
		"sw\t%2,20(%0)\n\t"
		"sw\t%2,24(%0)\n\t"
		"sw\t%2,28(%0)\n\t"
		"subu\t%1,1\n\t"
		"bnez\t%1,1b\n\t"
		"addiu\t%0,32\n\t"
		".set\treorder"
		:"=r" (dummy1),
		 "=r" (dummy2)
		:"r" (((unsigned long) invalid_pte_table - PAGE_OFFSET) |
		       _PAGE_TABLE),
		 "0" (page),
		 "1" (PAGE_SIZE/(sizeof(pmd_t)*8)),
		 "i" (Create_Dirty_Excl_D));
#endif
	/*
	 * Now force writeback to ashure values are in the RAM.
	 */
#ifndef CONF_DISABLE_KSEG0_CACHING
	sys_cacheflush((void *)page, PAGE_SIZE, DCACHE);
#endif
}
