/*
 * linux/include/asm-mips/ptrace.h
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994, 1995 by Waldorf GMBH
 * written by Ralf Baechle
 *
 * Machine dependent structs and defines to help the user use
 * the ptrace system call.
 */
#ifndef __ASM_MIPS_PTRACE_H
#define __ASM_MIPS_PTRACE_H

#include <asm/register.h>

/*
 * This struct defines the way the registers are stored on the stack during a
 * system call/exception. As usual the registers k0/k1 aren't being saved.
 */
struct pt_regs {
	/*
	 * Pad bytes for argument save space on the stack
	 * 20/40 Bytes for 32/64 bit code
	 */
	__register_t pad0[6];	/* extra word for alignment */

	/*
	 * Saved main processor registers
	 */
	__register_t	regs[32];

	/*
	 * Other saved registers
	 */
	__register_t	lo;
	__register_t	hi;
	__register_t	orig_reg2;

	/*
	 * saved cp0 registers
	 */
	__register_t	cp0_epc;
	__register_t	cp0_badvaddr;
	unsigned int	cp0_status;
	unsigned int	cp0_cause;
};

#ifdef __KERNEL__

/*
 * Does the process account for user or for system time?
 */
#if defined (__R4000__)

#define user_mode(regs) ((regs)->cp0_status & 0x10)

#else /* !defined (__R4000__) */

#define user_mode(regs) (!((regs)->cp0_status & 0x8))

#endif /* !defined (__R4000__) */

#define instruction_pointer(regs) ((regs)->cp0_epc)
extern void show_regs(struct pt_regs *);
#endif

#endif /* __ASM_MIPS_PTRACE_H */
